@@ -8,9 +8,15 @@
 /* Bug report URL. */
 #define BUG_REPORT_URL "http://llvm.org/bugs/"
 
+/* Default linker to use. */
+#define CLANG_DEFAULT_LINKER ""
+
 /* Default C++ stdlib to use. */
 #define CLANG_DEFAULT_CXX_STDLIB ""
 
+/* Default runtime library to use. */
+#define CLANG_DEFAULT_RTLIB ""
+
 /* Default OpenMP runtime used by -fopenmp. */
 #define CLANG_DEFAULT_OPENMP_RUNTIME "libomp"
 
@@ -32,8 +38,14 @@
 /* Define if we have libxml2 */
 #define CLANG_HAVE_LIBXML 1
 
+/* Define if we have z3 and want to build it */
+/* #undef CLANG_ANALYZER_WITH_Z3 */
+
+/* Define if we have sys/resource.h (rlimits) */
+#define CLANG_HAVE_RLIMITS 1
+
 /* The LLVM product name and version */
-#define BACKEND_PACKAGE_STRING "LLVM 3.9.1"
+#define BACKEND_PACKAGE_STRING "LLVM 5.0.2"
 
 /* Linker version detected at compile time. */
 /* #undef HOST_LINK_VERSION */
@@ -44,4 +56,9 @@
 /* enable x86 relax relocations by default */
 #define ENABLE_X86_RELAX_RELOCATIONS 0
 
+/* Enable each functionality of modules */
+#define CLANG_ENABLE_ARCMT
+#define CLANG_ENABLE_OBJC_REWRITER
+#define CLANG_ENABLE_STATIC_ANALYZER
+
 #endif
